#include "param.h"

float4   cnt    : register(c0);
sampler  t0     : register(s0);

float3 rotate_pos(float3 d, float rot) {
  float3x3 mtx = {
    cos(rot),  sin(rot), 0,
    sin(rot), -cos(rot), 0,
    0,       0,      1
  };
  d = mul(mtx, d).zxy;
  return mul(mtx, d).xzy;
}


float i_iso(float3 p) {
  p.x += sin(p.z) * 0.24;
  p.y += cos(p.z) * 0.27;
  float3 d = ( abs(p) % 1) - 0.5;
  if( (cnt.x % 20) > 10) return length(d.xy) - 0.15;
  return cos(p.x) + cos(p.y) + cos(p.z);
}


//
// perlin (http://www.pouet.net/topic.php?which=7920&page=18&x=31&y=14)
//
#define pi 3.14159265358979
inline float perlin(float3 p) {
  float3 i = floor(p);
  float4 a = dot(i, float3(1.0, 57.0, 21.0)) + float4(0.0, 57.0, 21.0, 78.0);
  float3 f = cos( (p - i) * pi) * (-0.5) + 0.5;
  a = lerp(sin(cos(a) * a), sin(cos(1.0 + a) * (1.0 + a)), f.x);
  a.xy = lerp(a.xz, a.yw, f.y);
  return lerp(a.x, a.y, f.z);
}

float iso(float3 p) {
  float t = i_iso(p);
  return t + (perlin(p * 1.7) * 0.3) + perlin(p * 20) * 0.01;
  //return i_iso(p) + perlin(p * 2) * 0.3 + perlin(perlin(p * 30) * 0.3) * 0.015;
  //return i_iso(p) + perlin(perlin(p * 30) * 0.3) * 0.015;
  //return i_iso(p) + perlin(perlin(p * 0.3) * 50) * 0.01;
}


//raycast
float raycast(float3 eye, float3 dir) {
  float t = 0;
  for(int i = 0; i < 45; i++) {
    float3 temp = eye + dir * t;
    float g = iso(temp);
    t += g * 0.95;
  }
  return t;
}

//get normal
float3 getnormal(float3 ip) {
  float  iit    = iso(ip);
  float3 h      = float3(0.01, 0, 0);
  return normalize(float3(
    iit - iso(ip + h.xyy),
    iit - iso(ip + h.yxy),
    iit - iso(ip + h.yyx)));
}

float4 ps_main(float2 vp : vpos) : color {
  float3 light  = normalize(float3(-0.7,  0.5,  0.7));
  float2 uv = -1 + 2 * vp / (float2(ScreenX, ScreenY));
  float time  = cnt.x;
  float rotate = (time + sin(time * 0.5)) * 0.1;
  float3x3 mtx = {
    cos(rotate),  sin(rotate), 0,
    sin(rotate), -cos(rotate), 0,
    0,            0,           1
  };
  
  //rotate
  float3 dir    = normalize(
    float3(uv.x * 1.25 - sin(time * 3) * 0.01,
           uv.y        + cos(time * 2) * 0.01,
      0.75));
  if( (time % 20) > 10) dir = mul(mtx, dir).yzx;
  dir = mul(mtx, dir).yzx;
  dir = mul(mtx, dir);
  
  //cast
  float3 eye    = float3(0, 0, cnt.x);
  float  t      = raycast(eye, dir);
  float3 normal = dot(light, getnormal(eye + dir * t));
  float3 rgb    = max(float3(0.7, 0.5, 0.3) * normal, 0) + dir * 0.3;
  float4 col    = 0;
  
  //fog
  col    = float4(rgb + (t * 0.07), 1);
  return float4(col.xyz, t) * min((time % 10) * 0.75, 1);
}


float4 ps_main_f(float2 vp : vpos, float2 uv : texcoord0) : color {
  return tex2D(t0, uv);
}


